#version 330 core

layout(location=0) out vec4 outColor;

in vec3 worldNormal;
in vec2 texCoord;
in vec3 worldPos;

struct Light
{
    vec3 AmbientColor;
    float AmbientIntensity;
    vec3 DiffuseDirection;
    float DiffuseIntensity;
}; 

uniform Light lighting;

uniform sampler2D color_texture;

uniform vec3 cameraPos;

uniform float SpecularIntensity;
uniform float SpecularPower;

void main() {

    vec4 AmbientColor = vec4(lighting.AmbientColor, 1.0f) * lighting.AmbientIntensity;


    vec3 Normal = normalize(worldNormal);
    float DiffuseFactor = dot(Normal, normalize(-lighting.DiffuseDirection));

    vec4 DiffuseColor = vec4(0, 0, 0, 0);
    vec4 SpecularColor = vec4(0, 0, 0, 0);

    if (DiffuseFactor > 0) {
        DiffuseColor = vec4(lighting.AmbientColor, 1.0f) * lighting.DiffuseIntensity * DiffuseFactor;

        vec3 VertexToCam = normalize( cameraPos - worldPos );
        vec3 LightReflect = normalize( reflect( normalize(lighting.DiffuseDirection), Normal ) );
        
        float SpecularFactor = dot( VertexToCam, LightReflect);
        SpecularFactor = pow( SpecularFactor, SpecularPower);

        if (SpecularFactor > 0) {
            SpecularColor = vec4( lighting.AmbientColor, 1.0f) * SpecularIntensity * SpecularFactor;
        }
    }
    
    outColor = texture2D(color_texture, texCoord.xy);
    //outColor = texture2D(color_texture, texCoord.xy) * (AmbientColor + DiffuseColor + SpecularColor);
    //outColor = vec4(0.7f,0.8f,0.6f,1.0f) * (AmbientColor + DiffuseColor + SpecularColor);
    //outColor = vec4(1.0f,1.0f,1.0f,1.0f);
}